@@ -58,9 +58,7 @@ class TestITListener(object):
 
             if parser.has_option('testit', 'testrunID'):
                 self.testrun_id = uuid_check(search_in_environ(
-                    parser.get('testit',
-                               'testrunID')
-                ))
+                    parser.get('testit', 'testrunID')))
             else:
                 self.project_id = uuid_check(
                     search_in_environ(parser.get('testit', 'projectID')))
@@ -69,9 +67,15 @@ class TestITListener(object):
         self.requests = Api(url, private_token, proxy=proxy)
 
     @pytest.hookimpl
-    def pytest_collection_modifyitems(self, session, items):
+    def pytest_collection_modifyitems(self, session, config, items):
         index = 0
         new_items = []
+        plugin_info = config.pluginmanager.list_plugin_distinfo()
+        for plugin, dist in plugin_info:
+            if 'pytest_check' == dist.project_name:
+                from pytest_check import check_methods
+                self.pytest_check_get_failures = check_methods.get_failures
+                break
 
         if hasattr(self, 'testrun_id'):
             self.project_id, data_autotests = self.requests.get_testrun(
@@ -90,17 +94,17 @@ class TestITListener(object):
                 if item.own_markers:
                     for mark in item.own_markers:
                         if mark.name == 'parametrize':
-                            if not hasattr(item, 'array_parametrize_id'):
-                                item.array_parametrize_id = []
-                            item.array_parametrize_id.append(
+                            if not hasattr(item, 'array_parametrize_mark_id'):
+                                item.array_parametrize_mark_id = []
+                            item.array_parametrize_mark_id.append(
                                 item.own_markers.index(mark))
 
                 item.test_external_id = self.param_attribute_collector(
                     item.function.test_external_id,
                     item.own_markers,
-                    item.array_parametrize_id,
+                    item.array_parametrize_mark_id,
                     index) if hasattr(item,
-                                      'array_parametrize_id'
+                                      'array_parametrize_mark_id'
                                       ) else item.function.test_external_id
 
                 item.index = index
@@ -172,8 +176,15 @@ class TestITListener(object):
     @pytest.hookimpl
     def pytest_runtest_logreport(self, report):
         if self.item:
-            if report.failed:
-                self.item.test_traces = report.longreprtext
+            if report.failed or hasattr(report, 'wasxfail') \
+                    and not report.passed or report.outcome == 'rerun':
+                if not hasattr(self.item, 'test_traces'):
+                    self.item.test_traces = ''
+                self.item.test_traces += (report.longreprtext if report.longreprtext else
+                    ''.join(['\n'.join(self.item.old_failures)]) +
+                    f'\n{"-" * 60}\nFailed Checks: {len(self.item.old_failures)}') + f'\n{"-" * 60}\n'
+                if hasattr(self.item, 'old_failures'):
+                    del self.item.old_failures
             if not hasattr(self.item, 'test_duration'):
                 self.item.test_duration = 0
             self.item.test_duration += report.duration * 1000
@@ -280,49 +291,65 @@ class TestITListener(object):
     @staticmethod
     def _get_setup_from(item):
         if hasattr(item, 'test_method_setup_steps'):
-            return item.test_method_setup_steps
+            method_setup_steps = item.test_method_setup_steps
+            del item.test_method_setup_steps
+            return method_setup_steps
         return []
 
     @staticmethod
     def _get_setup_results_from(item):
-        if hasattr(item, 'test_method_setup_steps'):
-            return item.test_method_setup_results_steps
+        if hasattr(item, 'test_method_setup_results_steps'):
+            method_setup_results_steps = item.test_method_setup_results_steps
+            del item.test_method_setup_results_steps
+            return method_setup_results_steps
         return []
 
     @staticmethod
     def _get_teardown_from(item):
         if hasattr(item, 'test_method_teardown_steps'):
-            return item.test_method_teardown_steps
+            method_teardown_steps = item.test_method_teardown_steps
+            del item.test_method_teardown_steps
+            return method_teardown_steps
         return []
 
     @staticmethod
     def _get_teardown_results_from(item):
         if hasattr(item, 'test_method_teardown_results_steps'):
-            return item.test_method_teardown_results_steps
+            method_teardown_results_steps = item.test_method_teardown_results_steps
+            del item.test_method_teardown_results_steps
+            return method_teardown_results_steps
         return []
 
     @staticmethod
     def _get_result_links_from(item):
         if hasattr(item, 'result_links'):
-            return item.result_links
+            result_links = item.result_links
+            del item.result_links
+            return result_links
         return []
 
     @staticmethod
     def _get_duration_from(item):
         if hasattr(item, 'test_duration'):
-            return item.test_duration
+            test_duration = item.test_duration
+            del item.test_duration
+            return test_duration
         return None
 
     @staticmethod
     def _get_traces_from(item):
         if hasattr(item, 'test_traces'):
-            return item.test_traces
+            test_traces = item.test_traces
+            del item.test_traces
+            return test_traces
         return None
 
     @staticmethod
     def _get_attachments_from(item):
         if hasattr(item, 'test_attachments'):
-            return item.test_attachments
+            test_attachments = item.test_attachments
+            del item.test_attachments
+            return test_attachments
         return []
 
     @staticmethod
@@ -340,28 +367,28 @@ class TestITListener(object):
 
     @staticmethod
     def _set_links(item, data):
-        if hasattr(item, 'array_parametrize_id'):
+        if hasattr(item, 'array_parametrize_mark_id'):
             for link in item.function.test_links:
                 data['links'].append({})
                 data['links'][-1]['url'] = TestITListener.attribute_collector_links(
                                                 link,
                                                 'url',
                                                 item.own_markers,
-                                                item.array_parametrize_id,
+                                                item.array_parametrize_mark_id,
                                                 item.index)
                 if link['title']:
                     data['links'][-1]['title'] = TestITListener.attribute_collector_links(
                                                     link,
                                                     'title',
                                                     item.own_markers,
-                                                    item.array_parametrize_id,
+                                                    item.array_parametrize_mark_id,
                                                     item.index)
                 if link['type']:
                     data['links'][-1]['type'] = TestITListener.attribute_collector_links(
                                                     link,
                                                     'type',
                                                     item.own_markers,
-                                                    item.array_parametrize_id,
+                                                    item.array_parametrize_mark_id,
                                                     item.index)
                 if link['description']:
                     data['links'][-1][
@@ -369,7 +396,7 @@ class TestITListener(object):
                                             link,
                                             'description',
                                             item.own_markers,
-                                            item.array_parametrize_id,
+                                            item.array_parametrize_mark_id,
                                             item.index)
         else:
             data['links'] = item.function.test_links
@@ -378,10 +405,10 @@ class TestITListener(object):
     def _get_title_from(item):
         if not hasattr(item.function, 'test_title'):
             return None
-        if hasattr(item, 'array_parametrize_id'):
+        if hasattr(item, 'array_parametrize_mark_id'):
             return TestITListener.param_attribute_collector(item.function.test_title,
                                                             item.own_markers,
-                                                            item.array_parametrize_id,
+                                                            item.array_parametrize_mark_id,
                                                             item.index)
         return item.function.test_title
 
@@ -389,21 +416,21 @@ class TestITListener(object):
     def _get_description_from(item):
         if not hasattr(item.function, 'test_description'):
             return None
-        if hasattr(item, 'array_parametrize_id'):
+        if hasattr(item, 'array_parametrize_mark_id'):
             return TestITListener.param_attribute_collector(
                 item.function.test_description,
                 item.own_markers,
-                item.array_parametrize_id,
+                item.array_parametrize_mark_id,
                 item.index)
         return item.function.test_description
 
     @staticmethod
     def _set_labels(item, data):
-        if hasattr(item, 'array_parametrize_id'):
+        if hasattr(item, 'array_parametrize_mark_id'):
             result, param_id = TestITListener.mass_param_attribute_collector(
                 item.function.test_labels[0],
                 item.own_markers,
-                item.array_parametrize_id,
+                item.array_parametrize_mark_id,
                 item.index)
             if param_id is not None and item.function.test_labels[0][1:-1] in \
                     item.name[(item.name.find('[') + 1):(item.name.rfind(']'))].split(
@@ -424,10 +451,10 @@ class TestITListener(object):
 
     @staticmethod
     def _set_workitems_id(item, data):
-        if hasattr(item, 'array_parametrize_id'):
+        if hasattr(item, 'array_parametrize_mark_id'):
             result, param_id = TestITListener.mass_param_attribute_collector(
                 item.function.test_workitems_id[0], item.own_markers,
-                item.array_parametrize_id, item.index)
+                item.array_parametrize_mark_id, item.index)
             if param_id is not None and item.function.test_workitems_id[0][
                                         1:-1] in \
                     item.name[(item.name.find('[') + 1):(item.name.rfind(']'))].split(
@@ -455,6 +482,10 @@ class TestITListener(object):
                 item),
             'resultLinks': TestITListener._get_result_links_from(item),
             'duration': TestITListener._get_duration_from(item),
+            'testResult': 'Failed' if hasattr(item,
+                                              'test_traces') else 'Passed',
+            'failureReasonName': 'TestDefect' if hasattr(item,
+                                                         'test_traces') else 'NoDefect',
             'traces': TestITListener._get_traces_from(item),
             'namespace': item.function.__module__,
             'attachments': TestITListener._get_attachments_from(item),
@@ -462,10 +493,6 @@ class TestITListener(object):
             'classname': TestITListener._get_classname_from(item),
             'title': TestITListener._get_title_from(item),
             'description': TestITListener._get_description_from(item),
-            'testResult': 'Failed' if hasattr(item,
-                                              'test_traces') else 'Passed',
-            'failureReasonName': 'TestDefect' if hasattr(item,
-                                                         'test_traces') else 'NoDefect',
             'links': [],
             'labels': [],
             'workItemsID': [],
@@ -483,8 +510,7 @@ class TestITListener(object):
 
         if item.own_markers:
             for mark in item.own_markers:
-                if mark.name == 'skip' or mark.name == 'skipif' and mark.args[
-                    0]:
+                if mark.name == 'skip' or mark.name == 'skipif' and mark.args[0]:
                     data['testResult'] = 'Skipped'
                     data['failureReasonName'] = None
                     if mark.kwargs:
@@ -494,11 +520,11 @@ class TestITListener(object):
             data['message'] = item.test_message
 
         if hasattr(item.function, 'test_displayname'):
-            if hasattr(item, 'array_parametrize_id'):
+            if hasattr(item, 'array_parametrize_mark_id'):
                 data['autoTestName'] = TestITListener.param_attribute_collector(
                     item.function.test_displayname,
                     item.own_markers,
-                    item.array_parametrize_id,
+                    item.array_parametrize_mark_id,
                     item.index)
             else:
                 data['autoTestName'] = item.function.test_displayname
@@ -508,8 +534,11 @@ class TestITListener(object):
             data['autoTestName'] = 'Test without name'
             data['testResult'] = 'Failed'
             data['failureReasonName'] = 'TestDefect'
-            data[
-                'traces'] = f'>\n{inspect.getsource(item.function)}\nE {item.originalname} must have @testit.displayName or documentation!\n{item.location[0]}:{item.location[1]}: Exception'
+            data['traces'] = '>\n{}\nE {} must have @testit.displayName or documentation!\n{}:{}: Exception'.format(
+                inspect.getsource(item.function),
+                item.originalname,
+                item.location[0],
+                item.location[1])
 
         return data
 
@@ -601,3 +630,12 @@ class TestITListener(object):
             self.item.test_parameters = {}
             for key, parameter in parameters.items():
                 self.item.test_parameters[key] = str(parameter)
+
+    @testit_pytest.hookimpl
+    def get_pytest_check_outcome(self):
+        if hasattr(self, 'pytest_check_get_failures'):
+            failures = self.pytest_check_get_failures()
+            if failures and ((not hasattr(self.item, 'old_failures')) or self.item.old_failures != failures):
+                self.item.old_failures = failures[:]
+                return 'Failed'
+        return 'Passed'
